def parse(string):
    is_key = False
    result = {}
    start_entry = 0
    start_key = 0
    stop_key = 0
    braces = 0
    for i, c in enumerate(string):
        if braces == 0:
            if c == "@":
                start_entry = i
                continue
            if c == "{":
                start_key = i + 1
                is_key = True
                braces += 1
                continue
        if is_key and c == ",":
            stop_key = i
            is_key = False
            continue
        if c == "{":
            braces += 1
        if c == "}":
            braces -= 1
            if braces == 0:
                key = string[start_key:stop_key]
                result[key] = string[start_entry : i + 1]
    return result



from pathlib import Path
import re
from typing import Dict, Set, List
import requests
from .io import parse
import os


def load(f) -> Dict:
    return parse(f.read())


def dump(dict, f):
    for entry in dict.values():
        f.write(f"{entry}\n\n")


def get_aux_bibdata(aux: Path) -> List[Path]:
    with open(aux) as f:
        names = re.findall(r"\\bibdata{([^}]+)}", f.read())
    # split comma-separated entries
    names = sum((name.split(",") for name in names), [])
    # make unique while preserving order
    dir = aux.parent
    return [dir / f"{name}.bib" for name in dict.fromkeys(names)]


def get_aux_citations(aux: Path) -> Set:
    with open(aux) as f:
        txt = f.read()
        tmp = re.findall(r"\\citation{([^}]+)}", txt)
    # split multi-citations
    result = set()
    for c in tmp:
        result.update(c.split(","))
    return result


def get_entry_online(key) -> Dict:
    # https://github.com/inspirehep/rest-api-doc
    r = requests.get(
        "https://inspirehep.net/api/literature", params={"q": key, "format": "bibtex"}
    )
    return parse(r.content.decode())


def find_in_path(name):
    paths = ["/bin", "/usr/bin", "/usr/local/bin"]
    for path in os.environ["PATH"].split(":"):
        path = os.path.expandvars(path)
        paths.append(path)

    results = []
    for path in paths:
        for p in Path(os.path.expandvars(path)).glob(name):
            results.append(p)

    return results



__version__ = "0.2.0"


def main():
    import sys
    from pathlib import Path
    from . import util
    import subprocess as subp

    args = sys.argv[1:]

    def log(x):
        print(f"autobib: {x}")

    print(f"autobib {__version__} called with args: {' '.join(args)}")
    if len(args) == 1:
        # bibtex accepts filename without extension
        aux = Path(args[0]).with_suffix(".aux")
        if aux.exists():
            log(f"Found {aux}")
            bib_files = util.get_aux_bibdata(aux)
            citations = util.get_aux_citations(aux)

            main = {}
            all = {}
            for bib in bib_files:
                if bib.exists():
                    with open(bib) as f:
                        db = util.load(f)
                        if not main:
                            main = db
                        all.update(db)

            bib = bib_files[0]

            unknown = citations - set(all.keys())

            if unknown:
                for c in unknown:
                    log(f"Fetching online: {c}")
                    ref = util.get_entry_online(c)
                    if not ref:
                        log(f"Warning: no entry found for '{c}'")
                    main.update(ref)
                # could be optimized so that only new entries are written
                log(f"Writing {bib}")
                with open(bib, "w") as f:
                    util.dump(main, f)

    # now run original bibtex
    bibtexs = util.find_in_path("bibtex")
    assert len(bibtexs) > 1
    for bibtex in bibtexs:
        with open(bibtex, "rb") as f:
            if f.read(2) == b"#!":
                continue
        break

    subp.run([bibtex] + args)
